/******************************************************************************
*
*  FILENAME :	file.c
*
*  DESCRIPTION :
*
*	Generic package for file manipulation functions.
*
*  PUBLIC FUNCTIONS :
*
*	void close_file(const char *filename, const FILE *fp)
*	FILE *open_file(const char *filename, const char *mode)
*	FILE *reopen_file(const char *filename, const char *mode, FILE *stream)
*
*	void append_file(const char *filename1, const char *filename2)
*	void copy_file(const char *filename1, const char *filename2)
*	void delete_file(const char *filename)
*	void print_file(const char *filename)
*	void read_file(const char *filename, const char *mode)
*
*	Boolean diff_file(const char *filename1, const char *filename2)
*	char *get_dir(const char *filename)
*
*  NOTES :
*
*	Functions cause program termination with appropriate message,
*	if an error occurs.
*
*	Written by Paul Chesson (1994)
*	E-Mail: chesson@mundil.cs.mu.OZ.AU
*
*/


/******************************************************************************
*  INCLUDE FILES
******************************************************************************/

#include <errno.h>
#include <stdio.h>

#include "bugbyte.h"


/******************************************************************************
*  PRIVATE DECLARATIONS
******************************************************************************/

static void acton_file(const char *filename1, const char *filename2,
		       const char *mode);


/******************************************************************************
*  PUBLIC FUNCTIONS
******************************************************************************/

/******************************************************************************
*
*  NAME :	void close_file(const char *filename, const FILE *stream)
*
*  DESCRIPTION :
*
*	Closes a file.
*
*  INPUTS :
*
*	PARAMETERS:	filename	Name of file
*			stream		File stream
*		
*/

void close_file(const char *filename, const FILE *stream)
{
    if (fclose((FILE *)stream) == EOF)
	FATAL(filename);
}


/******************************************************************************
*
*  NAME :	FILE *open_file(const char *filename, const char *mode)
*
*  DESCRIPTION :
*
*	Opens a file.
*
*  INPUTS :
*
*	PARAMETERS:	filename	Name of file
*			mode		Properties of file to be opened
*					(see fopen(3S))
*		
*  OUTPUTS :
*
*	RETURN:		stream of opened file
*
*/

FILE *open_file(const char *filename, const char *mode)
{
    FILE *fp;

    if ((fp = fopen(filename, mode)) == NULL)
	FATAL(filename);
    return fp;
}


/******************************************************************************
*
*  NAME :	FILE *reopen_file(const char *filename, const char *mode,
*				  FILE *stream)
*
*  DESCRIPTION :
*
*	Opens a file, associating a stream with it.
*
*  INPUTS :
*
*	PARAMETERS:	filename	Name of file to be opened
*			mode		Properties of file to be opened
*					(see fopen(3S))
*			stream		File stream to be associated with
*					opened file
*		
*  OUTPUTS :
*
*	RETURN:		stream of opened file
*
*/


FILE *reopen_file(const char *filename, const char *mode, FILE *stream)
{
    FILE *fp;

    if ((fp = freopen(filename, mode, stream)) == NULL)
	FATAL(filename);
    return fp;
}


/******************************************************************************
*
*  NAME :	void append_file(const char *filename1, const char *filename2)
*
*  DESCRIPTION :
*
*	Appends one file onto another.
*
*  INPUTS :
*
*	PARAMETERS:	filename1	Name of file to be appended onto
*			filename2	Name of file to be appended
*
*/

void append_file(const char *filename1, const char *filename2)
{
    acton_file(filename1, filename2, "a");
}


/******************************************************************************
*
*  NAME :	void copy_file(const char *filename1, const char *filename2)
*
*  DESCRIPTION :
*
*	Copies one file to another.
*
*  INPUTS :
*
*	PARAMETERS:	filename1	Name of file to be replaced
*			filename2	Name of file to be copied
*
*/

void copy_file(const char *filename1, const char *filename2)
{
    acton_file(filename1, filename2, "w+");
}


/******************************************************************************
*
*  NAME :	void delete_file(const char *filename)
*
*  DESCRIPTION :
*
*	Deletes a file.
*
*  INPUTS :
*
*	PARAMETERS:	filename	Name of file to be deleted
*
*/

void delete_file(const char *filename)
{
    if (remove(filename) < 0)
	FATAL(filename);
}


/******************************************************************************
*
*  NAME :	void write_file(const char *filename)
*
*  DESCRIPTION :
*
*	Writes a file to the standard output stream.
*
*  INPUTS :
*
*	PARAMETERS:	filename	Name of file to be written
*
*/

void write_file(const char *filename)
{
    const FILE *fp = open_file(filename, "r");
    char buf[BUFSIZ];

    while(fgets(buf, BUFSIZ, (FILE *)fp) != NULL)
    {
	if (fputs(buf, stdout) < 0)
	    FATAL(filename);
    }
	
    close_file(filename, fp);
}


/******************************************************************************
*
*  NAME :	void read_file(const char *filename, const char *mode)
*
*  DESCRIPTION :
*
*	Reads the standard input stream, and writes it to a file.
*
*  INPUTS :
*
*	PARAMETERS:	filename	Name of file to be printed
*			mode		Properties of file to be opened
*					(see fopen(3S))
*
*  NOTES :
*
*	Mode should be write or append.
*
*/

void read_file(const char *filename, const char *mode)
{
    const FILE *fp = open_file(filename, mode);
    char buf[BUFSIZ];

    while(fgets(buf, BUFSIZ, stdin) != NULL)
    {
	if (fputs(buf, (FILE *)fp) < 0)
	    FATAL(filename);
    }
    close_file(filename, fp);
}


/******************************************************************************
*
*  NAME :	char *get_dir(const char *filename)
*
*  DESCRIPTION :
*
*	Determines the name of a directory for a file.
*
*  INPUTS :
*
*	PARAMETERS:	filename	Name of file
*		
*  OUTPUTS :
*
*	RETURN:		String holding name of directory.
*
*  NOTES :
*
*	Returned name will relative or absolute depending on the
*	path given in the filename.
*
*	Returned name is held statically, so should be copied if
*	required for later use.
*
*/

char *get_dir(const char *filename)
{
    static char	dirname[FILENAME_MAX+1];
    char	*cp;

    strcpy(dirname, filename);
    cp = dirname + strlen(dirname);
    while (cp != dirname && *(cp-1) != '/')
	cp--;
    if (cp == dirname)
	strcpy(dirname, "./");
    else
	*cp = NUL;

    return dirname;
}


/******************************************************************************
*
*  NAME :	Boolean diff_file(const char *filename1, const char *filename2)
*
*  DESCRIPTION :
*
*	Determines if two files are different.
*
*  INPUTS :
*
*	PARAMETERS:	filename1	} name of files to be compared
*			filename2	}
*		
*  OUTPUTS :
*
*	RETURN:		TRUE		files are different
*			FALSE		otherwise
*
*/

Boolean diff_file(const char *filename1, const char *filename2)
{
    const FILE *fp1 = open_file(filename1, "r");
    const FILE *fp2 = open_file(filename2, "r");
    char buf1[BUFSIZ], buf2[BUFSIZ];
    Boolean result = FALSE;

    while((fgets(buf1, BUFSIZ, (FILE *)fp1) != NULL) && (result == FALSE))
    {
	if (fgets(buf2, BUFSIZ, (FILE *)fp2) == NULL)
	    result = TRUE;		/* file 1 is longer */
	if (STR_DIFF(buf1, buf2))
	    result = TRUE;
    }

    if (fgets(buf2, BUFSIZ, (FILE *)fp2) != NULL)
	result = TRUE;			/* file 2 is longer */

    close_file(filename1, fp1);
    close_file(filename2, fp2);
    return result;
}


/******************************************************************************
*  PRIVATE FUNCTIONS
******************************************************************************/

/******************************************************************************
*
*  NAME :	void acton_file(const char *filename1, const char *filename2,
*				const char *mode)
*
*  DESCRIPTION :
*
*	Generic function for writing or appending one file to another.
*
*  INPUTS :
*
*	PARAMETERS:	filename1	Name of file to be written to
*			filename2	Name of file to be read
*			mode		Properties of file to be opened
*					(see fopen(3S))
*
*  NOTES :
*
*	Mode should be write or append.
*		
*/

static void acton_file(const char *filename1, const char *filename2,
		       const char *mode)
{
    const FILE *fp1 = open_file(filename1, mode);
    const FILE *fp2 = open_file(filename2, "r");
    char buf[BUFSIZ];

    while(fgets(buf, BUFSIZ, (FILE *)fp2) != NULL)
    {
	if (fputs(buf, (FILE *)fp1) < 0)
	    FATAL(filename1);
    }

    close_file(filename1, fp1);
    close_file(filename2, fp2);
}

